/*
   FileComponent defines for a File-Handle a graphical representation.

   Copyright (C) 1997  Martin Vogt

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation.

   For more information look at the file COPYRIGHT in this package

*/



import java.awt.*;
import java.awt.event.*;
import java.io.*;
import TextMarkComponent;
import ImageFileComponent;
import de.unikl.awtnode.cfContainer;



/**
 * This class defines for a given File-Handle a graphical
 * representation.
 *
 * Im allgemeinen hat man ein verschiedenes "Aussehen" eines Files
 * So soll ein Directory mit einem "Ordner" IMage versehen werden
 * und ein .TXT File mit einem kleines Document, oder dem Icon der
 * Applictation mit dem es "verbunden" ist.
 *
 * Diese Klasse entscheidet an dem gegebenen File-Objekt, wie
 * die graphische Representation des File-Objekts aussehen soll.
 *
 * Ein File Objekt hat viele Attribute, die graphisch dargestellt werden
 * koennen.
 * <p><pre>
 * Der Name (evt mit Teilpfad, gesamtpfad).
 * Die groesse
 * Ein Icon fuer das File
 * die Zugriffsrechte
 * ob es noch vorhanden ist.
 *     Anm: es File das fuer exists() == false liefer
 *          ist nicht vorhanden, man muesste hier eine andere graphische
 *          darstellung waehlen
 * Anlegedatum
 * Zugriffsdatum
 * Aenderungsdatum
 * </pre><p>
 *
 * Man muss sicherlich eine Auswahlmaske definieren um festzulegen
 * welche Attribute sichtbar sind und welche nicht.
 * Die Componente die die Attribute sichtbar machen werden in
 * einem cfContainer gespeichert.
 *
 */



public class FileComponent  extends cfContainer
                            implements ActionListener,
                                       ItemListener {

  private static final String base = "filecomponent";
  private static int nameCounter = 0;


  /**
   * for this file we construct a graphical representation
   */
  File file;


  /**
   * Sometimes there are different images dependig whether
   * a file is open or not
   */
   boolean lopen;

  /**
   * stored for efficeny reasons
   **/

  TextMarkComponent nameComp;



  /**
   * stored for efficeny reasons
   **/

  ImageFileComponent imgComp;


  /**
   * Constructs a new FileComponent
   *
   * Currently we have only for the Name and the image
   * of a Filename a graphical description
   * @param file for this file we create a graphical representation
   * @lopen if true select an "open"-image otherwise a "close"-image
   */


  FileComponent(File file,boolean lopen) {
    this.file=file;
    this.lopen=lopen;
    String fname=file.getName();
    imgComp=new ImageFileComponent(file,lopen);
    add("image",imgComp);
    nameComp=new TextMarkComponent(fname);
    nameComp.addItemListener(this);
    add("name",nameComp);

    setLayout(new FlowLayout(FlowLayout.LEFT,5,0));
    setName(base + nameCounter++);
  }


  /**
   * @returns the File-Object which we represent graphically
   */
  public File getFile() {
    return file;
  }

  /**
   * @returns insets with default vaules : (0,-5,0,-5)
   */

  public Insets getInsets() {
     return new Insets(0,-5,0,-5);
  }


    /**
     * returns the status of the icon
     * Sometimes you need different icons for "opened" and "closed"
     * (e.g. a directory folder)
     * In the variable lopen this class stores the state of the icon
     */
     public boolean getOpen() {
        return lopen;
     }

     /**
      * sets the status for the icon
      * @param lopen true means : choose a "open" icon
      */
      public void setOpen(boolean lopen) {
        this.lopen=lopen;
      }




   /**
     * Returns the String representation of this NodeContainer values.
     */
    public String toString() {
        return getClass().getName() + "[" + nameComp.toString()+","
	                                  + file.toString()+"]";

    }


    // Interface : ActionListener[START]

    /**
     * Invoked when an action occurs.
     */
    public void actionPerformed(ActionEvent e) {
        String command=e.getActionCommand();
        System.out.println("in :"+getName());
        System.out.println("commad:"+command);
        if (command == "open") {
            remove(imgComp);
            imgComp=new ImageFileComponent(file,true);
            add(imgComp,0);
            setIdentifier("image",imgComp);
            lopen=true;

        } else if (command == "close") {
            remove(imgComp);
            imgComp=new ImageFileComponent(file,false);
            add(imgComp,0);
            setIdentifier("image",imgComp);
            lopen=false;

        }
    }


  // Interface : ActionListener[END]


  // Interface : ItemListener [START]

  /**
   * we implement the Standard behavior if a TextMarkComponent is
   * clicked
   */
  public void itemStateChanged(ItemEvent e) {
    if (e.getStateChange() == ItemEvent.SELECTED) {
        nameComp.setMark(true);
    } else {
        nameComp.setMark(false);
    }
  }

  // Interface : ItemListener [END]

}
