/*
   A component which  displays an Image.
   Copyright (C) 1997  Martin Vogt

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation.                                       

   For more information look at the file COPYRIGHT in this package

*/





import java.awt.*;
import java.lang.*;
import ImageServer;


/**
 * A component that displays an Image.
 * Note: The componets gets fully loaded before its displayed
 *
 * we use a static variable to specify the search path for images
 *
 */





public class ImageComponent extends Canvas {




  /**
   * The image.
   */
  Image image;

  String name;

    /**
     * Constructs an empty Component. No image is displayed
     */
    public ImageComponent() {
	this("");
    }


    /**
     * Constructs a new Image.
     * The image is specified with its name
     * @param path the path to the image
     */
    public ImageComponent (String name) {
      setImage(name);
    }



  public Dimension getPreferredSize() {
    Dimension back=new Dimension(0,0);
    if (image != null){
      back.height=image.getHeight(this);
      back.width=image.getWidth(this);
    }
    return back;
  }


  public Dimension getMinimumSize() {
    return getPreferredSize();
  }




  public void paint(Graphics g) {

    if (image != null) {
      g.drawImage(image,0,0,this);
    }


  }



  /**
   * Set the image, specified with the image name
   *
   */
  public void setImage(String name) {
    this.name=name;
    this.image=ImageServer.getImage(name,this);

  }




    /**
     * Returns the String representation of this NodeContainer values.
     */
    public String toString() {
        return getClass().getName() + "[name="+name+"]";

    }



}

