/*
   cfContainer store componets and an identifier for each Component
   Copyright (c) 1997 Fraunhofer-Einrichtung fuer Experimentelles
                      Software Engineering (IESE)

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation.

   For more information look at the file COPYRIGHT in this package

*/

package de.unikl.awtnode;


import java.awt.*;
import java.util.*;




/**
<p>
Documentation of the implemenation of cfContainer
<p>

<em>Classification mechanism:</em><p>



Like an ordinary Container the cfContainer contains Components.
The only thing which is unusal is that you can pass with each component
a String. With this String each component gets its identifier.
<p>
<em>For what is this good for?</em><p>

In a Container stands components. Components are painted on the screen
and to do so you need a LayoutManager.
The standard LayoutManager doesnt distinguish between the components.
e.g: FlowLayout lays out all the Components from left to right, and
GridLayout orders them in rows and colums. But they dont give a
component a "position" on the screen according to the *kind* of
component during the layout-operation.
In fact there is a LayoutManger which does this: Borderlayout.
But if you look at the source code, the Layoutmanager itsself stores
the "kind" of component. In BorderLayout there a 5 possible kinds
of Components:
<p>
 "north","south","east","west" and "center".
<p>
But because of the assignment in the LayoutManager itsself
you cannot easily replace the LayoutManager with a new one:
<p>
Example:
<pre><code>
	setLayout(new BorderLayout());   // 1
	add("Center",mycomp);
	setLayout(new BorderLayout());	//  2
</code></pre>
doesnt work, because the second BorderLayouManager doesnt "know"
about the "pointer" of mycomp.
Thus the attempt to store the compoents in the LayoutManger itsself
leads to problems if you replace the LayouManager.
As a result these "kinds" of Components have to be stored in
the container, and the Implemenation of cfContainer does this
with the "identifiers".(a simple String for each component)
cf stands for [C]lassi[F]ied
<p>
Thus you can set for each component you insert its identifier.You can
pass the Identifier in the add-method or seperatly (after the
Component is already added) with the method:

<pre><code>

setIdentifier(String name,Component comp)

</code></pre>
you can retrieve this identifier with:
<code><pre>

getIdentifier(Component comp)

</pre></code>
<p>

The LayoutManager for a cfContainer may use this identifer to
assign a special position for the component or not.



**/



public class cfContainer extends Panel {

  /**
   * this stores the "identifiers" of each component
   * in the cfContainer.
   *
   * The key of the HashTable is the Component-Pointer
   * The value of the HashTable ist the identifier (as String)
   */
   Hashtable identifiers;

  /**
   * Constructs a new [c]lassi[f]ied Container, with an empty
   * identifier list.
   */

  public cfContainer() {
    super();
    identifiers=new Hashtable();
  }


  /**
   * Adds the specified component to this container.
   * @param comp the component to be added
   */

  public Component add(Component  comp) {

    // if the String is not yet registered the we add it
    if (isContained(comp) == false) {
      identifiers.put(comp,"");
      return super.add(comp,-1);
    }
    return comp;
  }

  /**
   * Adds the specified component to this container at the given position.
   * The Identifer added is the empty String ""
   * @param comp the component to be added
   * @param pos the position at which to insert the component. -1
   * means insert at the end.
   * @see #remove
   */

  public Component add(Component  comp, int  pos) {

    // if the String is not yet registered the we add it
    if (isContained(comp) == false) {
      identifiers.put(comp,"");
      return super.add(comp,pos);
    }
    return comp;
  }

  /**
   * Adds the specified component to this container.
   * The identifier added is the <name> arugument.The component
   * is also added to the layout manager of this container using the
   * name specified
   * @param name the component name
   * @param comp the component to be added
   * @see #remove
   * @see LayoutManager
   */
  public Component add(String  name, Component  comp) {

    // if the String is not yet registered the we add it
    if (isContained(comp) == false) {
      identifiers.put(comp,name);
      return super.add(comp,-1);
    }
    return comp;
  }


  /**
   * Removes the specified component from this container.
   * The Identifer is removed also.
   * @param comp the component to be removed
   * @see #add
   */
  public void remove(Component  comp) {
    identifiers.remove(comp);
    super.remove(comp);
  }



  /**
   * Removes all the components from this container.
   * The Identifer list is cleared
   * @see #add
   * @see #remove
   */
  public void removeAll() {
    identifiers.clear();
    super.removeAll();
  }



    /**
     * returns the position of the component in the container
     * @return the index of the Coomponent, or -1 if not contained
     */
     public int getIndex(Component comp) {
        int i;
        int n=getComponentCount();
        for(i=0;i<n;i++) {
            if (getComponent(i)==comp) return i;
        }
        return -1;
     }

  /**
   * returns the identifer of the Component (as String)
   * or null, if Component does not belong to the Container!
   *
   */

  public String getIdentifier(Component m) {
    Object identifier;
    if ((identifier=identifiers.get(m)) != null) return (String)identifier;
    return null;
  }

  /**
   * returns true if the Component is already in the Container,
   * otherwise false
   */

  public boolean isContained (Component comp) {
    if (identifiers.get(comp) == null) return false;
    return true;
  }


  /**
   * Sets the identifer for a specified Component. If Component
   * is not contained in cfContainer, nothing is done!
   */

  public void setIdentifier(String name,Component comp) {
    if (isContained(comp)==true) {
      int i=getIndex(comp);
      identifiers.remove(comp);
      identifiers.put(comp,name);
    }
  }


  /**
   * This method return the number of Components which have
   * the specified filter String as idetifier
   *
   * @param filtered components must have this identifer
   * @returns number of Components which have the <filter> as identifier
   */

  public final int getComponentCount(String filter) {
    int i;
    int back=0;
    int n=getComponentCount();
    Component comp[]=getComponents();

    for(i=0;i<n;i++) {
      if (((String)identifiers.get(comp[i])).equals(filter)) { //opt
	back++;
      }
    }
    return back;
  }



  /**
   * With this method you have a set-oriented Funktion to
   * access all Components which match the specific Filter expression
   *
   * @param filtered components must have this identifer
   * @returns all the Component which have the String filter as identifier
   */

  /* Note: this method is speed-optimized
   * to do: implemtent an ordered identifier-Array
   * (maximum speed)
   */
  public final Component[] getComponents(String filter) {
    int i;
    Component back[];
    int n=getComponentCount();
    Component comp[] = getComponents();

    int matchSize=0;
    Component matchArray[] = new Component[n];

    for(i=0;i<n;i++) {
      if (((String)identifiers.get(comp[i])).equals(filter)) { //opt
 	matchArray[matchSize]=comp[i];
	matchSize++;
      }
    }
    back=new Component[matchSize];
    System.arraycopy(matchArray,0,back,0,matchSize);
    return back;
  }


  /**
   * This method removes all Components which have the specified
   * filter String as identifier
   *
   */
  public void removeAll(String filter) {
    int i;
    Component comp[]=getComponents(filter);
    for(i=0;i<comp.length;i++) remove(comp[i]);
  }


    private String ComponentInfo(int i) {
        String back="";
        Component m=getComponent(i);
        back=back+"{identifier="+getIdentifier(m);
        back=back+",Component="+m.toString()+"}";
        return back;
    }

    public String toString() {
        int i;
        int n=getComponentCount();
        String back;
        back= getClass().getName() + "[";
        for(i=0;i<n;i++) {
            back=back+ComponentInfo(i);
            if (i+1 < n) back=back+",";
        }
        return back+"]";
    }
}









